# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long, protected-access, too-few-public-methods
from knack.log import get_logger
from azure.cli.core.azclierror import ArgumentUsageError
from azure.cli.core.aaz import register_command, AAZResourceIdArgFormat, has_value, AAZListArg, AAZResourceIdArg, \
    AAZStrArg, AAZArgEnum
from azure.mgmt.core.tools import is_valid_resource_id, parse_resource_id
from ..aaz.latest.network.lb import Delete as _LBDelete, Update as _LBUpdate, List as _LBList, Show as _LBShow
from ..aaz.latest.network.lb.frontend_ip import Create as _LBFrontendIPCreate, Update as _LBFrontendIPUpdate, \
    Show as _LBFrontendIPShow, Delete as _LBFrontendIPDelete, List as _LBFrontendIPList
from ..aaz.latest.network.lb.inbound_nat_pool import Create as _LBInboundNatPoolCreate, \
    Update as _LBInboundNatPoolUpdate
from ..aaz.latest.network.lb.inbound_nat_rule import Create as _LBInboundNatRuleCreate, \
    Update as _LBInboundNatRuleUpdate
from ..aaz.latest.network.lb.rule import Create as _LBRuleCreate, Update as _LBRuleUpdate, Show as _LBRuleShow, \
    Delete as _LBRuleDelete, List as _LBRuleList
from ..aaz.latest.network.lb.outbound_rule import Create as _LBOutboundRuleCreate, Update as _LBOutboundRuleUpdate
from ..aaz.latest.network.lb.address_pool import Create as _LBAddressPoolCreate, Update as _LBAddressPoolUpdate, \
    Show as _LBAddressPoolShow, Delete as _LBAddressPoolDelete, List as _LBAddressPoolList
from ..aaz.latest.network.lb.address_pool.address import Add as _LBAddressPoolAddressAdd, \
    Update as _LBAddressPoolAddressUpdate, Show as _LBAddressPoolAddressShow, \
    Remove as _LBAddressPoolAddressRemove, List as _LBAddressPoolAddressList
from ..aaz.latest.network.lb.address_pool.basic import Create as _LBAddressPoolBasicCreate, \
    Delete as _LBAddressPoolBasicDelete
from ..aaz.latest.network.lb.address_pool.tunnel_interface import Add as _LBAddressPoolTunnelInterfaceAdd, \
    Update as _LBAddressPoolTunnelInterfaceUpdate
from ..aaz.latest.network.lb.probe import Create as _LBProbeCreate, Update as _LBProbeUpdate


logger = get_logger(__name__)


class LBFrontendIPCreate(_LBFrontendIPCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet_name = AAZStrArg(
            arg_group="Properties",
            options=['--vnet-name'],
            help="The virtual network (VNet) associated with the subnet (Omit if supplying a subnet id)."
        )
        args_schema.subnet._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{vnet_name}/subnets/{}",
        )
        args_schema.public_ip_prefix._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIpPrefixes/{}",
        )
        args_schema.public_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIPAddresses/{}",
        )
        args_schema.gateway_lb._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{}/frontendIPConfigurations/{}"
        )

        args_schema.zones.Element.enum = AAZArgEnum({
            "1": "1",
            "2": "2",
            "3": "3",
        })
        args_schema.private_ip_allocation_method._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.subnet) and has_value(args.public_ip_address):
            raise ArgumentUsageError(
                'incorrect usage: --subnet NAME --vnet-name NAME | '
                '--subnet ID | --public-ip-address NAME_OR_ID')

        if not has_value(args.public_ip_address):
            logger.warning(
                "Please note that the default public IP used for LB frontend will be changed from Basic to Standard "
                "in the future."
            )

        if has_value(args.private_ip_address):
            args.private_ip_allocation_method = 'Static'
        else:
            args.private_ip_allocation_method = 'Dynamic'


class LBFrontendIPUpdate(_LBFrontendIPUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet_name = AAZStrArg(
            arg_group="Properties",
            options=['--vnet-name'],
            help="The virtual network (VNet) associated with the subnet (Omit if supplying a subnet id)."
        )
        args_schema.subnet._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{vnet_name}/subnets/{}",
        )
        args_schema.public_ip_prefix._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIpPrefixes/{}",
        )
        args_schema.public_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIPAddresses/{}",
        )
        args_schema.gateway_lb._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{}/frontendIPConfigurations/{}"
        )

        args_schema.zones.Element.enum = AAZArgEnum({
            "1": "1",
            "2": "2",
            "3": "3",
        })
        args_schema.private_ip_allocation_method._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.private_ip_address):
            # update private_ip_address
            if args.private_ip_address:
                args.private_ip_allocation_method = 'Static'
            else:
                # set private_ip_address as null value
                args.private_ip_allocation_method = 'Dynamic'

    def post_instance_update(self, instance):
        if not has_value(instance.properties.subnet.id):
            instance.properties.subnet = None
        if not has_value(instance.properties.public_ip_address.id):
            instance.properties.public_ip_address = None
        if not has_value(instance.properties.public_ip_prefix.id):
            instance.properties.public_ip_prefix = None
        if not has_value(instance.properties.gateway_load_balancer.id):
            instance.properties.gateway_load_balancer = None


class LBInboundNatPoolCreate(_LBInboundNatPoolCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        return args_schema

    def pre_instance_create(self):
        args = self.ctx.args
        if not has_value(args.frontend_ip_name):
            instance = self.ctx.vars.instance
            frontend_ip_configurations = instance.properties.frontend_ip_configurations
            if len(frontend_ip_configurations) == 1:
                args.frontend_ip_name = instance.properties.frontend_ip_configurations[0].id
            elif len(frontend_ip_configurations) > 1:
                raise ArgumentUsageError("Multiple FrontendIpConfigurations found in loadbalancer. Specify --frontend-ip explicitly.")


class LBInboundNatPoolUpdate(_LBInboundNatPoolUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        return args_schema

    def post_instance_update(self, instance):
        if not has_value(instance.properties.frontend_ip_configuration.id):
            instance.properties.frontend_ip_configuration = None


class LBInboundNatRuleCreate(_LBInboundNatRuleCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        args_schema.backend_address_pool._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
        )

        # required for a public load balancer
        args_schema.protocol._required = True
        args_schema.backend_port._required = True
        return args_schema

    def pre_instance_create(self):
        args = self.ctx.args
        if not has_value(args.frontend_ip_name):
            instance = self.ctx.vars.instance
            frontend_ip_configurations = instance.properties.frontend_ip_configurations
            if len(frontend_ip_configurations) == 1:
                args.frontend_ip_name = instance.properties.frontend_ip_configurations[0].id
            elif len(frontend_ip_configurations) > 1:
                raise ArgumentUsageError("Multiple FrontendIpConfigurations found in loadbalancer. Specify --frontend-ip explicitly.")


class LBInboundNatRuleUpdate(_LBInboundNatRuleUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        args_schema.backend_address_pool._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
        )
        return args_schema

    def post_instance_update(self, instance):
        if not has_value(instance.properties.frontend_ip_configuration.id):
            instance.properties.frontend_ip_configuration = None
        if not has_value(instance.properties.backend_address_pool.id):
            instance.properties.backend_address_pool = None


class LBRuleCreate(_LBRuleCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        args_schema.probe_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/probes/{}"
        )
        # list argument accept one element: `--backend-pool-name PoolName`
        args_schema.backend_pools = AAZListArg(
            options=["--backend-pools-name", "--backend-pool-name"],
            arg_group="Properties",
            help="List of ID or name of the backend address pools. Multiple pools are only supported by Gateway SKU load balancer. If only one exists, omit to use as default."
        )
        args_schema.backend_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
            )
        )

        args_schema.protocol._required = True
        args_schema.frontend_port._required = True
        args_schema.backend_port._required = True
        args_schema.backend_address_pools._registered = False
        return args_schema

    def pre_instance_create(self):
        from azure.cli.core.aaz.utils import assign_aaz_list_arg
        args = self.ctx.args
        if not has_value(args.frontend_ip_name):
            instance = self.ctx.vars.instance
            frontend_ip_configurations = instance.properties.frontend_ip_configurations
            if len(frontend_ip_configurations) == 1:
                args.frontend_ip_name = instance.properties.frontend_ip_configurations[0].id
            elif len(frontend_ip_configurations) > 1:
                raise ArgumentUsageError(
                    "Multiple FrontendIpConfigurations found in loadbalancer. Specify --frontend-ip explicitly.")
        if not has_value(args.backend_pools):
            instance = self.ctx.vars.instance
            backend_address_pools = instance.properties.backend_address_pools
            if len(backend_address_pools) == 1:
                args.backend_pools = [instance.properties.backend_address_pools[0].id]
            elif len(backend_address_pools) > 1:
                raise ArgumentUsageError(
                    "Multiple BackendAddressPools found in loadbalancer. Specify --backend-pool-name explicitly.")
        args.backend_address_pools = assign_aaz_list_arg(
            args.backend_address_pools, args.backend_pools,
            element_transformer=lambda _, id: {"id": id}
        )

    def post_instance_create(self, instance):
        args = self.ctx.args
        if has_value(args.frontend_ip_name):
            curr_id = args.frontend_ip_name.to_serialized_data()
            curr_name = parse_resource_id(curr_id)["resource_name"] if is_valid_resource_id(curr_id) else curr_id

            parent = self.ctx.vars.instance
            frontend_ip_configurations = parent.properties.frontend_ip_configurations
            for fip in frontend_ip_configurations:
                if fip.name == curr_name:
                    if has_value(fip.properties.gateway_load_balancer):
                        rid = fip.properties.gateway_load_balancer.id.to_serialized_data()
                        self.ctx.update_aux_subscriptions(parse_resource_id(rid)["subscription"])


class LBRuleUpdate(_LBRuleUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        args_schema.probe_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/probes/{}"
        )

        args_schema.backend_pools = AAZListArg(
            options=["--backend-pools-name", "--backend-pool-name"],
            nullable=True,
            arg_group="Properties",
            help="List of ID or name of the backend address pools. Multiple pools are only supported by Gateway SKU load balancer."
        )

        args_schema.backend_pools.Element = AAZResourceIdArg(
            nullable=True,
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
            )
        )

        args_schema.protocol._nullable = False
        args_schema.frontend_port._nullable = False
        args_schema.backend_port._nullable = False
        args_schema.backend_address_pools._registered = False
        return args_schema

    def pre_operations(self):
        from azure.cli.core.aaz.utils import assign_aaz_list_arg
        args = self.ctx.args
        args.backend_address_pools = assign_aaz_list_arg(
            args.backend_address_pools, args.backend_pools,
            element_transformer=lambda _, id: {"id": id}
        )

    def post_instance_update(self, instance):
        if not has_value(instance.properties.frontend_ip_configuration.id):
            instance.properties.frontend_ip_configuration = None
        if not has_value(instance.properties.probe.id):
            instance.properties.probe = None
        # always remove backend_address_pool in update request, service will fill this property based on backend_address_pools property.
        instance.properties.backend_address_pool = None

        args = self.ctx.args
        if has_value(args.frontend_ip_name):
            curr_id = args.frontend_ip_name.to_serialized_data()
            curr_name = parse_resource_id(curr_id)["resource_name"] if is_valid_resource_id(curr_id) else curr_id

            parent = self.ctx.vars.instance
            frontend_ip_configurations = parent.properties.frontend_ip_configurations
            for fip in frontend_ip_configurations:
                if fip.name == curr_name:
                    if has_value(fip.properties.gateway_load_balancer):
                        rid = fip.properties.gateway_load_balancer.id.to_serialized_data()
                        self.ctx.update_aux_subscriptions(parse_resource_id(rid)["subscription"])


class LBOutboundRuleCreate(_LBOutboundRuleCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.backend_address_pool._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
        )
        args_schema.frontend_ip_configs = AAZListArg(
            options=["--frontend-ip-configs"],
            arg_group="Properties",
            help="The List of frontend IP configuration IDs or names.",
        )
        args_schema.frontend_ip_configs.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
            )
        )

        args_schema.protocol._required = True
        args_schema.backend_address_pool._required = True
        args_schema.frontend_ip_configurations._registered = False
        return args_schema

    def pre_operations(self):
        from azure.cli.core.aaz.utils import assign_aaz_list_arg
        args = self.ctx.args
        args.frontend_ip_configurations = assign_aaz_list_arg(
            args.frontend_ip_configurations,
            args.frontend_ip_configs,
            element_transformer=lambda _, id: {"id": id}
        )


class LBOutboundRuleUpdate(_LBOutboundRuleUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.backend_address_pool._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
        )
        args_schema.frontend_ip_configs = AAZListArg(
            options=["--frontend-ip-configs"],
            arg_group="Properties",
            help="The List of frontend IP configuration IDs or names.",
        )
        args_schema.frontend_ip_configs.Element = AAZResourceIdArg(
            nullable=True,
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
            )
        )

        args_schema.protocol._nullable = False
        args_schema.backend_address_pool._nullable = False
        args_schema.frontend_ip_configurations._registered = False
        return args_schema

    def pre_operations(self):
        from azure.cli.core.aaz.utils import assign_aaz_list_arg
        args = self.ctx.args
        args.frontend_ip_configurations = assign_aaz_list_arg(
            args.frontend_ip_configurations,
            args.frontend_ip_configs,
            element_transformer=lambda _, id: {"id": id}
        )


@register_command("network lb address-pool create")
class LBAddressPoolCreate(_LBAddressPoolBasicCreate):
    """Create load balancer backend address pool.

    :example: Create an address pool.
        az network lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool

    :example: Create an address pool with several backend addresses using shorthand syntax arguments.
        az network lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool --vnet MyVnetResource --backend-addresses "[{name:addr1,ip-address:10.0.0.1},{name:addr2,ip-address:10.0.0.2,subnet:subnetName}]"

    :example: Create an address pool with several backend addresses using config file
        az network lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-addresses config_file.json

    :example: Create an address pool with one backend address using key-value arguments.
        az network lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-address name=addr1 ip-address=10.0.0.1 subnet=/subscriptions/000/resourceGroups/MyRg/providers/Microsoft.Network/virtualNetworks/vnet/subnets/subnet1
    """

    # inherient the BackendAddressPoolsCreateOrUpdate operation
    class LoadBalancerBackendAddressPoolsCreateOrUpdate(_LBAddressPoolCreate.LoadBalancerBackendAddressPoolsCreateOrUpdate):

        def on_200_201(self, session):
            # ignore the response data.
            pass

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet = AAZResourceIdArg(
            options=["--vnet"],
            arg_group="Properties",
            help="Name or Id of the default virtual network applied to backend addresses in `--backend-addresses`.",
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{}"
            )
        )

        args_schema.admin_state = AAZStrArg(
            options=["--admin-state"],
            arg_group="Properties",
            help="Default administrative state to backend addresses in `--backend-addresses`.",
        )
        args_schema.admin_state.enum = args_schema.backend_addresses.Element.admin_state.enum

        args_schema.backend_addresses.Element.virtual_network._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{}"
        )

        args_schema.backend_addresses.Element.name._required = True
        args_schema.backend_addresses.Element.ip_address._required = True
        args_schema.backend_addresses.Element.frontend_ip_address._registered = False
        args_schema.vnet_id._registered = False
        return args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.LoadBalancersGet(ctx=self.ctx)()
        self.pre_instance_create()
        sku = self.ctx.vars.instance.sku.name.to_serialized_data()
        if sku.lower() == "basic":
            self.InstanceCreateByJson(ctx=self.ctx)()
            self.post_instance_create(self.ctx.selectors.subresource.required())
            yield self.LoadBalancersCreateOrUpdate(ctx=self.ctx)()
        else:
            # use AddressPoolsCreateOrUpdate API to update Standarded or Geteway lb
            yield self.LoadBalancerBackendAddressPoolsCreateOrUpdate(ctx=self.ctx)()
            self.LoadBalancersGet(ctx=self.ctx)()
        self.post_operations()

    def pre_operations(self):
        from azure.cli.core.aaz import AAZUndefined

        args = self.ctx.args
        if has_value(args.sync_mode) and has_value(args.vnet):
            args.vnet_id = args.vnet
            args.vnet = AAZUndefined
        if has_value(args.backend_addresses):
            for backend_address in args.backend_addresses:
                if not has_value(backend_address.admin_state) and has_value(args.admin_state):
                    # use the command level argument --admin-state
                    backend_address.admin_state = args.admin_state

                virtual_network = backend_address.virtual_network.to_serialized_data()
                if not virtual_network and has_value(args.vnet):
                    # use the command level argument --vnet
                    virtual_network = args.vnet.to_serialized_data()
                    backend_address.virtual_network = virtual_network

                subnet = backend_address.subnet.to_serialized_data()
                if subnet and not is_valid_resource_id(subnet):
                    if not virtual_network:
                        raise ArgumentUsageError(
                            "Invalid backend address `{}`: vnet name or vnet ID is required when using subnet name only.".format(
                                backend_address.name)
                        )
                    # convert subnet name to subnet id
                    subnet = f"{virtual_network}/subnets/{subnet}"
                    backend_address.subnet = subnet

                if not virtual_network and not subnet:
                    raise ArgumentUsageError(
                        "Invalid backend address `{}`: vnet or subnet is required.".format(
                            backend_address.name)
                    )

    def pre_instance_create(self):
        args = self.ctx.args
        if not has_value(args.tunnel_interfaces):
            instance = self.ctx.vars.instance
            if has_value(instance.sku.name) and instance.sku.name.to_serialized_data().lower() == 'gateway':
                # when sku is 'gateway', 'tunnelInterfaces' can't be None. Otherwise, service will respond error
                args.tunnel_interfaces = [{"identifier": 900, "type": 'Internal', "protocol": 'VXLAN'}]


class LBAddressPoolUpdate(_LBAddressPoolUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet = AAZResourceIdArg(
            options=["--vnet"],
            arg_group="Properties",
            help="Name or Id of the default virtual network applied to backend addresses in `--backend-addresses`.",
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{}"
            )
        )
        args_schema.admin_state = AAZStrArg(
            options=["--admin-state"],
            arg_group="Properties",
            help="Default administrative state to backend addresses in `--backend-addresses`.",
        )
        args_schema.admin_state.enum = args_schema.backend_addresses.Element.admin_state.enum

        args_schema.backend_addresses.Element.virtual_network._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{}"
        )

        args_schema.backend_addresses.Element.name._nullable = False
        args_schema.backend_addresses.Element.ip_address._nullable = False
        args_schema.backend_addresses.Element.frontend_ip_address._registered = False
        args_schema.vnet_id._registered = False
        return args_schema

    def pre_operations(self):
        from azure.cli.core.aaz import AAZUndefined

        args = self.ctx.args
        if has_value(args.sync_mode) and has_value(args.vnet):
            args.vnet_id = args.vnet
            args.vnet = AAZUndefined
        if has_value(args.backend_addresses) and args.backend_addresses.to_serialized_data() is not None:
            for backend_address in args.backend_addresses:
                if not has_value(backend_address.admin_state) and has_value(args.admin_state):
                    # use the command level argument --admin-state
                    backend_address.admin_state = args.admin_state
                if not has_value(backend_address.virtual_network) and has_value(args.vnet):
                    # use the command level argument --vnet
                    backend_address.virtual_network = args.vnet
                subnet = backend_address.subnet.to_serialized_data()
                if subnet and not is_valid_resource_id(subnet):
                    virtual_network = backend_address.virtual_network.to_serialized_data()
                    if not virtual_network:
                        raise ArgumentUsageError(
                            "Invalid backend address: vnet name or vnet ID is required when using subnet name only."
                        )
                    # convert subnet name to subnet id
                    subnet = f"{virtual_network}/subnets/{subnet}"
                    backend_address.subnet = subnet

    def post_instance_update(self, instance):
        if has_value(instance.properties.load_balancer_backend_addresses):
            for backend_address in instance.properties.load_balancer_backend_addresses:
                if not has_value(backend_address.properties.virtual_network.id):
                    backend_address.properties.virtual_network = None
                if not has_value(backend_address.properties.subnet.id):
                    backend_address.properties.subnet = None


@register_command("network lb address-pool delete")
class LBAddressPoolDelete(_LBAddressPoolBasicDelete):
    """Delete the specified load balancer backend address pool.

    :example: Delete an address pool.
        az network lb address-pool delete -g MyResourceGroup --lb-name MyLb -n MyAddressPool
    """


class LBAddressPoolAddressAdd(_LBAddressPoolAddressAdd):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.virtual_network._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{}"
        )
        args_schema.ip_address._required = True
        args_schema.frontend_ip_address._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        virtual_network = args.virtual_network.to_serialized_data()
        subnet = args.subnet.to_serialized_data()
        if subnet and not is_valid_resource_id(subnet):
            if not virtual_network:
                raise ArgumentUsageError(
                    "vnet name or vnet ID is required when using subnet name only."
                )
            # convert subnet name to subnet id
            subnet = f"{virtual_network}/subnets/{subnet}"
            args.subnet = subnet

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result


class LBAddressPoolAddressUpdate(_LBAddressPoolAddressUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.virtual_network._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/virtualNetworks/{}"
        )
        args_schema.ip_address._nullable = False
        args_schema.frontend_ip_address._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        virtual_network = args.virtual_network.to_serialized_data()
        subnet = args.subnet.to_serialized_data()
        if subnet and not is_valid_resource_id(subnet):
            if not virtual_network:
                raise ArgumentUsageError(
                    "vnet name or vnet ID is required when using subnet name only."
                )
            # convert subnet name to subnet id
            subnet = f"{virtual_network}/subnets/{subnet}"
            args.subnet = subnet

    def post_instance_update(self, instance):
        if not has_value(instance.properties.virtual_network.id):
            instance.properties.virtual_network = None
        if not has_value(instance.properties.subnet.id):
            instance.properties.subnet = None

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result


class LBAddressPoolTunnelInterfaceAdd(_LBAddressPoolTunnelInterfaceAdd):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.identifier._required = True
        args_schema.type._required = True
        args_schema.protocol._required = True
        return args_schema

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result


class LBAddressPoolTunnelInterfaceUpdate(_LBAddressPoolTunnelInterfaceUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.identifier._nullable = False
        args_schema.type._nullable = False
        args_schema.protocol._nullable = False
        return args_schema

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result


class LBProbeCreate(_LBProbeCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.port._required = True
        args_schema.protocol._required = True
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.number_of_probes):
            logger.warning(
                "The property \"numberOfProbes\" is not respected. Load Balancer health probes will probe up or down "
                "immediately after one probe regardless of the property's configured value. To control the number of "
                "successful or failed consecutive probes necessary to mark backend instances as healthy or unhealthy, "
                "please leverage the property \"probeThreshold\" instead."
            )
        if has_value(args.request_path) and args.request_path == "":
            args.request_path = None


class LBProbeUpdate(_LBProbeUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.port._nullable = False
        args_schema.protocol._nullable = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.number_of_probes):
            logger.warning(
                "The property \"numberOfProbes\" is not respected. Load Balancer health probes will probe up or down "
                "immediately after one probe regardless of the property's configured value. To control the number of "
                "successful or failed consecutive probes necessary to mark backend instances as healthy or unhealthy, "
                "please leverage the property \"probeThreshold\" instead."
            )
        if has_value(args.request_path) and args.request_path == "":
            args.request_path = None


# cross-region-lb commands
@register_command("network cross-region-lb show")
class CrossRegionLoadBalancerShow(_LBShow):
    """Get the details of a load balancer.

    :example: Get the details of a load balancer.
        az network cross-region-lb show -g MyResourceGroup -n MyLb
    """


@register_command("network cross-region-lb delete")
class CrossRegionLoadBalancerDelete(_LBDelete):
    """Delete the specified load balancer.

    :example: Delete a load balancer.
        az network cross-region-lb delete -g MyResourceGroup -n MyLb
    """


@register_command("network cross-region-lb list")
class CrossRegionLoadBalancerList(_LBList):
    """List load balancers.

    :example: List load balancers.
        az network cross-region-lb list -g MyResourceGroup
    """


@register_command("network cross-region-lb update")
class CrossRegionLoadBalancerUpdate(_LBUpdate):
    """Update a load balancer.

    This command can only be used to update the tags for a load balancer.
    Name and resource group are immutable and cannot be updated.

    :example: Update the tags of a load balancer.
        az network cross-region-lb update -g MyResourceGroup -n MyLB --tags CostCenter=MyTestGroup
    """


@register_command("network cross-region-lb frontend-ip show")
class CrossRegionLoadBalancerFrontendIPShow(_LBFrontendIPShow):
    """Get the details of a frontend IP address.

    :example: Get the details of a frontend IP address.
        az network cross-region-lb frontend-ip show -g MyResourceGroup --lb-name MyLb -n MyFrontendIp
    """


@register_command("network cross-region-lb frontend-ip list")
class CrossRegionLoadBalancerFrontendIPList(_LBFrontendIPList):
    """List frontend IP addresses.

    :example: List frontend IP addresses.
        az network cross-region-lb frontend-ip list -g MyResourceGroup --lb-name MyLb
    """


@register_command("network cross-region-lb frontend-ip delete")
class CrossRegionLoadBalancerFrontendIPDelete(_LBFrontendIPDelete):
    """Delete a frontend IP address.

    :example: Delete a frontend IP address.
        az network cross-region-lb frontend-ip delete -g MyResourceGroup --lb-name MyLb -n MyFrontendIp
    """


@register_command("network cross-region-lb frontend-ip create")
class CrossRegionLoadBalancerFrontendIPCreate(_LBFrontendIPCreate):
    """Create a frontend IP address.

    :example: Create a frontend ip address for a public load balancer.
        az network cross-region-lb frontend-ip create -g MyResourceGroup --lb-name MyLb -n MyFrontendIp --public-ip-address MyFrontendIp
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.public_ip_prefix._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIpPrefixes/{}",
        )
        args_schema.public_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIPAddresses/{}",
        )
        args_schema.zones.Element.enum = AAZArgEnum({
            "1": "1",
            "2": "2",
            "3": "3",
        })

        args_schema.private_ip_address._registered = False
        args_schema.private_ip_address_version._registered = False
        args_schema.private_ip_allocation_method._registered = False
        args_schema.subnet._registered = False
        args_schema.gateway_lb._registered = False
        return args_schema


@register_command("network cross-region-lb frontend-ip update")
class CrossRegionLoadBalancerFrontendIPUpdate(_LBFrontendIPUpdate):
    """Update a frontend IP address.

    :example: Update the frontend IP address of a public load balancer.
        az network cross-region-lb frontend-ip update -g MyResourceGroup --lb-name MyLb -n MyFrontendIp --public-ip-address MyFrontendIp
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.public_ip_prefix._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIpPrefixes/{}",
        )
        args_schema.public_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/publicIPAddresses/{}",
        )
        args_schema.zones.Element.enum = AAZArgEnum({
            "1": "1",
            "2": "2",
            "3": "3",
        })

        args_schema.private_ip_address._registered = False
        args_schema.private_ip_address_version._registered = False
        args_schema.private_ip_allocation_method._registered = False
        args_schema.subnet._registered = False
        args_schema.gateway_lb._registered = False
        return args_schema


@register_command("network cross-region-lb rule show")
class CrossRegionLoadBalancerRuleShow(_LBRuleShow):
    """Get the details of a load balancing rule.

    :example: Get the details of a load balancing rule.
        az network cross-region-lb rule show -g MyResourceGroup --lb-name MyLb -n MyLbRule
    """


@register_command("network cross-region-lb rule delete")
class CrossRegionLoadBalancerRuleDelete(_LBRuleDelete):
    """Delete a load balancing rule.

    :example: Delete a load balancing rule.
        az network cross-region-lb rule delete -g MyResourceGroup --lb-name MyLb -n MyLbRule
    """


@register_command("network cross-region-lb rule list")
class CrossRegionLoadBalancerRuleList(_LBRuleList):
    """List load balancing rules.

    :example: List load balancing rules.
        az network cross-region-lb rule list -g MyResourceGroup --lb-name MyLb -o table
    """


@register_command("network cross-region-lb rule create")
class CrossRegionLoadBalancerRuleCreate(_LBRuleCreate):
    """Create a load balancing rule.

    :example: Create a load balancing rule that assigns a front-facing IP configuration and port to an address pool and port.
        az network cross-region-lb rule create -g MyResourceGroup --lb-name MyLb -n MyLbRule --protocol Tcp --frontend-ip-name MyFrontEndIp --frontend-port 80 --backend-pool-name MyAddressPool --backend-port 80

    :example: Create a load balancing rule that assigns a front-facing IP configuration and port to an address pool and port with the floating ip feature.
        az network cross-region-lb rule create -g MyResourceGroup --lb-name MyLb -n MyLbRule --protocol Tcp --frontend-ip-name MyFrontEndIp --frontend-port 80 --backend-pool-name MyAddressPool --backend-port 80 --floating-ip true

    :example: Create an HA ports load balancing rule that assigns a frontend IP and port to use all available backend IPs in a pool on the same port.
        az network cross-region-lb rule create -g MyResourceGroup --lb-name MyLb -n MyHAPortsRule --protocol All --frontend-port 0 --backend-port 0 --frontend-ip-name MyFrontendIp --backend-pool-name MyAddressPool
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        args_schema.probe_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/probes/{}"
        )
        # not support multi backend pools because the loadbalance SKU is not Gateway
        args_schema.backend_pool = AAZResourceIdArg(
            options=["--backend-pool-name"],
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
            ),
            arg_group="Properties",
            help="ID or name of the backend address pools. If only one exists, omit to use as default."
        )
        args_schema.idle_timeout_in_minutes._default = 4
        args_schema.enable_tcp_reset._default = False
        args_schema.protocol._required = True
        args_schema.frontend_port._required = True
        args_schema.backend_port._required = True
        args_schema.backend_address_pools._registered = False
        args_schema.disable_outbound_snat._registered = False   # it's not required for cross-region-lb
        args_schema.idle_timeout_in_minutes._registered = False
        args_schema.enable_tcp_reset._registered = False
        return args_schema

    def pre_instance_create(self):
        args = self.ctx.args
        if not has_value(args.frontend_ip_name):
            instance = self.ctx.vars.instance
            frontend_ip_configurations = instance.properties.frontend_ip_configurations
            if len(frontend_ip_configurations) == 1:
                args.frontend_ip_name = instance.properties.frontend_ip_configurations[0].id
            elif len(frontend_ip_configurations) > 1:
                raise ArgumentUsageError(
                    "Multiple FrontendIpConfigurations found in loadbalancer. Specify --frontend-ip explicitly.")
        if not has_value(args.backend_pool):
            instance = self.ctx.vars.instance
            backend_address_pools = instance.properties.backend_address_pools
            if len(backend_address_pools) == 1:
                args.backend_pool = instance.properties.backend_address_pools[0].id
            elif len(backend_address_pools) > 1:
                raise ArgumentUsageError(
                    "Multiple BackendAddressPools found in loadbalancer. Specify --backend-pool-name explicitly.")
        if has_value(args.backend_pool):
            args.backend_address_pools = [{"id": args.backend_pool.to_serialized_data()}]


@register_command("network cross-region-lb rule update")
class CrossRegionLoadBalancerRuleUpdate(_LBRuleUpdate):
    """Update a load balancing rule.

    :example:  Update a load balancing rule to change the protocol to UDP.
        az network cross-region-lb rule update -g MyResourceGroup --lb-name MyLb -n MyLbRule --protocol Udp

    :example: Update a load balancing rule to support HA ports.
        az network cross-region-lb rule update -g MyResourceGroup --lb-name MyLb -n MyLbRule --protocol All --frontend-port 0 --backend-port 0
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.frontend_ip_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/frontendIPConfigurations/{}"
        )
        args_schema.probe_name._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/probes/{}"
        )
        # not support multi backend pools because the loadbalance SKU is not Gateway
        args_schema.backend_pool = AAZResourceIdArg(
            options=["--backend-pool-name"],
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{lb_name}/backendAddressPools/{}"
            ),
            arg_group="Properties",
            nullable=True,
            help="ID or name of the backend address pools. If only one exists, omit to use as default."
        )

        args_schema.protocol._nullable = False
        args_schema.frontend_port._nullable = False
        args_schema.backend_port._nullable = False
        args_schema.backend_address_pools._registered = False
        args_schema.disable_outbound_snat._registered = False   # it's not required for cross-region-lb
        args_schema.idle_timeout_in_minutes._registered = False
        args_schema.enable_tcp_reset._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.backend_pool):
            backend_pool = args.backend_pool.to_serialized_data()
            if backend_pool is None:
                args.backend_address_pools = []  # remove backend pool
            else:
                args.backend_address_pools = [{"id": backend_pool}]

    def post_instance_update(self, instance):
        if not has_value(instance.properties.frontend_ip_configuration.id):
            instance.properties.frontend_ip_configuration = None
        if not has_value(instance.properties.probe.id):
            instance.properties.probe = None
        # always remove backend_address_pool in update request, service will fill this property based on backend_address_pools property.
        instance.properties.backend_address_pool = None


@register_command("network cross-region-lb address-pool create")
class CrossRegionLoadBalancerAddressPoolCreate(_LBAddressPoolCreate):
    """Create load balancer backend address pool.

    :example: Create an address pool.
        az network cross-region-lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool

    :example: Create an address pool with several backend addresses using shorthand syntax arguments.
        az network cross-region-lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-addresses "[{name:addr1,frontend-ip-address:'/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb1'},{name:addr2,frontend-ip-address:'/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb2'}]"

    :example: Create an address pool with several backend addresses using config file
        az network cross-region-lb address-pool create -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-addresses config_file.json
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.admin_state = AAZStrArg(
            options=["--admin-state"],
            arg_group="Properties",
            help="Default administrative state to backend addresses in `--backend-addresses`.",
        )
        args_schema.admin_state.enum = args_schema.backend_addresses.Element.admin_state.enum
        # not support name, the frontend id should belong to a regional load balance
        args_schema.backend_addresses.Element.frontend_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{}/frontendIPConfigurations/{}"
        )
        args_schema.backend_addresses.Element.name._required = True
        args_schema.backend_addresses.Element.frontend_ip_address._required = True

        args_schema.tunnel_interfaces._registered = False
        args_schema.backend_addresses.Element.ip_address._registered = False
        args_schema.backend_addresses.Element.subnet._registered = False
        args_schema.backend_addresses.Element.virtual_network._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.backend_addresses):
            for backend_address in args.backend_addresses:
                if not has_value(backend_address.admin_state) and has_value(args.admin_state):
                    # use the command level argument --admin-state
                    backend_address.admin_state = args.admin_state


@register_command("network cross-region-lb address-pool update")
class CrossRegionLoadBalancerAddressPoolUpdate(_LBAddressPoolUpdate):
    """Update a load balancer backend address pool.

    :example: Update all backend addresses in the address pool using shorthand syntax
        az network cross-region-lb address-pool update -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-addresses "[{name:addr1,frontend-ip-address:'/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb1'},{name:addr2,frontend-ip-address:'/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb2'}]"

    :example: Update the frontend-ip-address of the first backend address in the address pool using shorthand syntax
        az network cross-region-lb address-pool update -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-addresses [0].frontend-ip-address=/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb1

    :example: Remove the first backend address in the address pool using shorthand syntax
        az network cross-region-lb address-pool update -g MyResourceGroup --lb-name MyLb -n MyAddressPool --backend-addresses [0]=null
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.admin_state = AAZStrArg(
            options=["--admin-state"],
            arg_group="Properties",
            help="Default administrative state to backend addresses in `--backend-addresses`.",
        )
        args_schema.admin_state.enum = args_schema.backend_addresses.Element.admin_state.enum
        # not support name, the frontend id should belong to a regional load balance
        args_schema.backend_addresses.Element.frontend_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{}/frontendIPConfigurations/{}"
        )
        args_schema.backend_addresses.Element.name._nullable = False
        args_schema.backend_addresses.Element.frontend_ip_address._nullable = False
        args_schema.backend_addresses.Element.admin_state._nullable = False

        args_schema.tunnel_interfaces._registered = False
        args_schema.backend_addresses.Element.ip_address._registered = False
        args_schema.backend_addresses.Element.subnet._registered = False
        args_schema.backend_addresses.Element.virtual_network._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.backend_addresses):
            for backend_address in args.backend_addresses:
                if not has_value(backend_address.admin_state) and has_value(args.admin_state):
                    # use the command level argument --admin-state
                    backend_address.admin_state = args.admin_state


@register_command("network cross-region-lb address-pool show")
class CrossRegionLoadBalancerAddressPoolShow(_LBAddressPoolShow):
    """Get load balancer backend address pool.

    :example: Get the details of an address pool.
        az network cross-region-lb address-pool show -g MyResourceGroup --lb-name MyLb -n MyAddressPool
    """


@register_command("network cross-region-lb address-pool delete")
class CrossRegionLoadBalancerAddressPoolDelete(_LBAddressPoolDelete):
    """Delete the specified load balancer backend address pool.

    :example: Delete an address pool.
        az network cross-region-lb address-pool delete -g MyResourceGroup --lb-name MyLb -n MyAddressPool
    """


@register_command("network cross-region-lb address-pool list")
class CrossRegionLoadBalancerAddressPoolList(_LBAddressPoolList):
    """List all the load balancer backed address pools.

    :example: List address pools.
        az network cross-region-lb address-pool list -g MyResourceGroup --lb-name MyLb -o table
    """


@register_command("network cross-region-lb address-pool address add")
class CrossRegionLoadBalancerAddressPoolAddressAdd(_LBAddressPoolAddressAdd):
    """Add one backend address into the load balance backend address pool.

    :example: Add one backend address into the load balance backend address pool.
        az network cross-region-lb address-pool address add -g MyResourceGroup --lb-name MyLb --pool-name MyAddressPool -n MyAddress --frontend-ip-address /subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb2
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.frontend_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{}/frontendIPConfigurations/{}"
        )

        args_schema.frontend_ip_address._required = True
        args_schema.ip_address._registered = False
        args_schema.subnet._registered = False
        args_schema.virtual_network._registered = False
        return args_schema

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result


@register_command("network cross-region-lb address-pool address remove")
class CrossRegionLoadBalancerAddressPoolAddressRemove(_LBAddressPoolAddressRemove):
    """Remove one backend address from the load balance backend address pool.
    :example: Remove one backend address from the load balance backend address pool.
        az network cross-region-lb address-pool address remove -g MyResourceGroup --lb-name MyLb --pool-name MyAddressPool -n MyAddress
    """


@register_command("network cross-region-lb address-pool address update")
class CrossRegionLoadBalancerAddressPoolAddressUpdate(_LBAddressPoolAddressUpdate):
    """Update the backend address into the load balance backend address pool.

    :example: Update the frontend ip of the backend address into the load balance backend address pool.
        az network cross-region-lb address-pool address update -g MyResourceGroup --lb-name MyLb --pool-name MyAddressPool -n MyAddress --frontend-ip-address /subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/cli_test_lb_address_pool_addresses000001/providers/Microsoft.Network/loadBalancers/regional-lb/frontendIPConfigurations/fe-rlb2
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.frontend_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/loadBalancers/{}/frontendIPConfigurations/{}"
        )

        args_schema.frontend_ip_address._nullable = False
        args_schema.ip_address._registered = False
        args_schema.subnet._registered = False
        args_schema.virtual_network._registered = False
        return args_schema

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result


@register_command("network cross-region-lb address-pool address list")
class CrossRegionLoadBalancerAddressPoolAddressList(_LBAddressPoolAddressList):
    """List all backend addresses of the load balance backend address pool.

    :example: List all backend addresses of the load balance backend address pool.
        az network cross-region-lb address-pool address list -g MyResourceGroup --lb-name MyLb --pool-name MyAddressPool
    """


@register_command("network cross-region-lb address-pool address show")
class CrossRegionLoadBalancerAddressPoolAddressShow(_LBAddressPoolAddressShow):
    """Show the backend address from the load balance backend address pool.

    :example: Show the backend address from the load balance backend address pool.
        az network cross-region-lb address-pool address show -g MyResourceGroup --lb-name MyLb --pool-name MyAddressPool -n MyAddress
    """



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""
Logging for Azure CLI

- Loggers: The name of the parent logger is defined in CLI_LOGGER_NAME variable. All the loggers used in the CLI
           must descends from it, otherwise it won't benefit from the logger handlers, filters and level configuration.

- Handlers: There are two default handlers will be added to both CLI parent logger and root logger. One is a colorized
            stream handler for console output and the other is a file logger handler. The file logger can be enabled or
            disabled through 'az configure' command. The logging file locates at path defined in AZ_LOGFILE_DIR.

- Level: Based on the verbosity option given by users, the logging levels for root and CLI parent loggers are:

               CLI Parent                  Root
            Console     File        Console     File
omitted     Warning     Debug       Critical    Debug
--verbose   Info        Debug       Critical    Debug
--debug     Debug       Debug       Debug       Debug

"""

import os
import logging
import datetime

from azure.cli.core.commands.events import EVENT_INVOKER_PRE_CMD_TBL_TRUNCATE

from knack.events import EVENT_CLI_POST_EXECUTE
from knack.log import CLILogging, get_logger, LOG_FILE_ENCODING
from knack.util import ensure_dir


_UNKNOWN_COMMAND = "unknown_command"
_CMD_LOG_LINE_PREFIX = "CMD-LOG-LINE-BEGIN"


class AzCliLogging(CLILogging):
    COMMAND_METADATA_LOGGER = 'az_command_data_logger'

    def __init__(self, name, cli_ctx=None):
        super().__init__(name, cli_ctx)
        self.command_log_dir = os.path.join(cli_ctx.config.config_dir, 'commands')
        self.command_logger_handler = None
        self.command_metadata_logger = None
        self.cli_ctx.register_event(EVENT_INVOKER_PRE_CMD_TBL_TRUNCATE, AzCliLogging.init_command_file_logging)
        self.cli_ctx.register_event(EVENT_CLI_POST_EXECUTE, AzCliLogging.deinit_cmd_metadata_logging)

    def configure(self, args):
        super().configure(args)
        from knack.log import CliLogLevel
        if self.log_level == CliLogLevel.DEBUG:
            # As azure.core.pipeline.policies.http_logging_policy is a redacted version of
            # azure.core.pipeline.policies._universal, disable azure.core.pipeline.policies.http_logging_policy
            # when debug log is shown.
            logging.getLogger("azure.core.pipeline.policies.http_logging_policy").setLevel(logging.CRITICAL)

    def get_command_log_dir(self):
        return self.command_log_dir

    @staticmethod
    def init_command_file_logging(cli_ctx, **kwargs):
        def _delete_old_logs(log_dir):
            """
            Periodically delete the 5 oldest command log files, ensuring that only the history of the last
            25 commands (or less) are kept.
            """

            # get log file names and sort them from newest to oldest file.
            log_file_names = [file for file in os.listdir(log_dir) if file.endswith(".log")]
            sorted_files = sorted(log_file_names, reverse=True)

            # if we have too many files, delete the 5 last / oldest command log files.
            if len(sorted_files) > 25:
                for file in sorted_files[-5:]:
                    try:
                        os.remove(os.path.join(log_dir, file))
                    except OSError:  # FileNotFoundError introduced in Python 3
                        continue

        # if tab-completion and not command don't log to file.
        if not cli_ctx.data.get('completer_active', False):
            self = cli_ctx.logging
            args = kwargs['args']

            metadata_logger = logging.getLogger(AzCliLogging.COMMAND_METADATA_LOGGER)

            # Overwrite the default of knack.log.CLILogging._is_file_log_enabled() to True
            self.file_log_enabled = cli_ctx.config.getboolean('logging', 'enable_log_file', fallback=True)

            if self.file_log_enabled:
                self._init_command_logfile_handlers(metadata_logger, args)  # pylint: disable=protected-access
                _delete_old_logs(self.command_log_dir)

    def _init_command_logfile_handlers(self, command_metadata_logger, args):

        ensure_dir(self.command_log_dir)
        command = self.cli_ctx.invocation._rudimentary_get_command(args) or _UNKNOWN_COMMAND  # pylint: disable=protected-access, line-too-long
        command_str = command.replace(" ", "_")
        if command_str.lower() == "feedback":
            return

        date_str = str(datetime.datetime.now().date())
        time = datetime.datetime.now().time()
        time_str = "{:02}-{:02}-{:02}".format(time.hour, time.minute, time.second)

        log_name = "{}.{}.{}.{}.{}".format(date_str, time_str, command_str, os.getpid(), "log")

        log_file_path = os.path.join(self.command_log_dir, log_name)
        get_logger(__name__).debug("metadata file logging enabled - writing logs to '%s'.", log_file_path)

        logfile_handler = logging.FileHandler(log_file_path, encoding=LOG_FILE_ENCODING)

        lfmt = logging.Formatter(_CMD_LOG_LINE_PREFIX + ' %(process)d | %(asctime)s | %(levelname)s | %(name)s | %(message)s')  # pylint: disable=line-too-long
        logfile_handler.setFormatter(lfmt)
        logfile_handler.setLevel(logging.DEBUG)
        # command_metadata_logger should always accept all levels regardless of the root logger level.
        command_metadata_logger.setLevel(logging.DEBUG)
        command_metadata_logger.addHandler(logfile_handler)

        self.command_logger_handler = logfile_handler
        self.command_metadata_logger = command_metadata_logger

        args = AzCliLogging._get_clean_args(command if command != _UNKNOWN_COMMAND else None, args)
        command_metadata_logger.info("command args: %s", " ".join(args))

    @staticmethod
    def _get_clean_args(command, args):  # TODO: add test for this function
        # based on AzCliCommandInvoker._extract_parameter_names(args)
        # note: name start with more than 2 '-' will be treated as value e.g. certs in PEM format

        # if no command provided, try to guess the intended command. This does not work for positionals
        if not command:
            command_list = []
            for arg in args:
                if arg.startswith('-') and not arg.startswith('---') and len(arg) > 1:
                    break
                command_list.append(arg)
            command = " ".join(command_list)

        command = command.split()
        cleaned_args = []
        placeholder = "{}"
        for i, arg in enumerate(args):
            # while this token a part of the command add it.
            # Note: if 'command' is none first positional would be captured.
            if i < len(command):
                cleaned_args.append(arg)
                continue

            # if valid optional name
            if arg.startswith('-') and not arg.startswith('---') and len(arg) > 1:

                # if short option with or without "="
                if not arg.startswith("--"):
                    opt = arg[:2]  # get opt

                    opt = opt + "=" if len(arg) > 2 and arg[2] == "=" else opt  # append '=' if necessary
                    opt = opt + placeholder if (len(arg) > 2 and arg[2] != "=") or len(
                        arg) > 3 else opt  # append placeholder if argument with optional
                    cleaned_args.append(opt)
                    continue

                # otherwise if long option with "="
                if "=" in arg:
                    opt, _ = arg.split('=', 1)
                    cleaned_args.append(opt + "=" + placeholder)
                    continue

                cleaned_args.append(arg)
                continue

            # else if positional or optional argument / value
            cleaned_args.append(placeholder)

        return cleaned_args

    def log_cmd_metadata_extension_info(self, extension_name, extension_version):
        if self.command_metadata_logger:
            self.command_metadata_logger.info("extension name: %s", extension_name)
            self.command_metadata_logger.info("extension version: %s", extension_version)

    @staticmethod
    def deinit_cmd_metadata_logging(cli_ctx):
        cli_ctx.logging.end_cmd_metadata_logging(cli_ctx.result.exit_code if cli_ctx.result else 128)

    def end_cmd_metadata_logging(self, exit_code):  # leave it non '-' prefix to not to break user
        if self.command_metadata_logger:
            self.command_metadata_logger.info("exit code: %s", exit_code)

            # We have finished metadata logging, remove handler and set command_metadata_handler to None.
            # crucial to remove handler as in python logger objects are shared which can affect testing of this logger
            # we do not want duplicate handlers to be added in subsequent calls of _init_command_logfile_handlers
            self.command_logger_handler.close()
            self.command_metadata_logger.removeHandler(self.command_logger_handler)
            self.command_metadata_logger = None


class CommandLoggerContext:
    """A context manager during which error logs are also written to az_command_data_logger for
    `az feedback` usage.
    """
    def __init__(self, module_logger):
        metadata_logger = logging.getLogger(AzCliLogging.COMMAND_METADATA_LOGGER)
        original_error = module_logger.error

        # Duplicate error logging to metadata logger
        def error_duplicated(*args, **kwargs):
            original_error(*args, **kwargs)
            metadata_logger.error(*args, **kwargs)
        from unittest import mock
        self.mock_cm = mock.patch.object(module_logger, 'error', error_duplicated)

    def __enter__(self):
        self.mock_cm.__enter__()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.mock_cm.__exit__(exc_type, exc_val, exc_tb)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import logging
import re
import types

from azure.core.pipeline.policies import SansIOHTTPPolicy, UserAgentPolicy
from knack.log import get_logger

_LOGGER = get_logger(__name__)


class SafeNetworkTraceLoggingPolicy(SansIOHTTPPolicy):
    """The logging policy that redacts specified headers.
    Based on azure.core.pipeline.policies._universal.NetworkTraceLoggingPolicy
    """

    def __init__(self, headers_to_redact=None):
        """
        :param list[str] headers_to_redact: headers that should be redacted from the log.
          Default to 'Authorization', 'x-ms-authorization-auxiliary'.
        """
        if headers_to_redact is not None:
            self.headers_to_redact = headers_to_redact
        else:
            self.headers_to_redact = ['authorization', 'x-ms-authorization-auxiliary']

    def on_request(self, request):
        http_request = request.http_request
        options = request.context.options
        logging_enable = options.pop("logging_enable", True)
        request.context["logging_enable"] = logging_enable
        if logging_enable:
            if not _LOGGER.isEnabledFor(logging.DEBUG):
                return

            try:
                _LOGGER.debug("Request URL: %r", http_request.url)
                _LOGGER.debug("Request method: %r", http_request.method)
                _LOGGER.debug("Request headers:")
                for header, value in http_request.headers.items():
                    if header.lower() in self.headers_to_redact:
                        value = '*****'
                    _LOGGER.debug("    %r: %r", header, value)
                _LOGGER.debug("Request body:")

                # We don't want to log the binary data of a file upload.
                if isinstance(http_request.body, types.GeneratorType):
                    _LOGGER.debug("File upload")
                    return
                try:
                    if isinstance(http_request.body, types.AsyncGeneratorType):
                        _LOGGER.debug("File upload")
                        return
                except AttributeError:
                    pass
                if http_request.body:
                    _LOGGER.debug(str(http_request.body))
                    return
                _LOGGER.debug("This request has no body")
            except Exception as err:  # pylint: disable=broad-except
                _LOGGER.debug("Failed to log request: %r", err)

    def on_response(self, request, response):  # pylint: disable=unused-argument, no-self-use
        http_response = response.http_response
        try:
            logging_enable = response.context["logging_enable"]
            if logging_enable:
                if not _LOGGER.isEnabledFor(logging.DEBUG):
                    return

                _LOGGER.debug("Response status: %r", http_response.status_code)
                _LOGGER.debug("Response headers:")
                for res_header, value in http_response.headers.items():
                    _LOGGER.debug("    %r: %r", res_header, value)

                # We don't want to log binary data if the response is a file.
                _LOGGER.debug("Response content:")
                pattern = re.compile(r'attachment; ?filename=["\w.]+', re.IGNORECASE)
                header = http_response.headers.get('content-disposition')

                if header and pattern.match(header):
                    filename = header.partition('=')[2]
                    _LOGGER.debug("File attachments: %s", filename)
                elif http_response.headers.get("content-type", "").endswith("octet-stream"):
                    _LOGGER.debug("Body contains binary data.")
                elif http_response.headers.get("content-type", "").startswith("image"):
                    _LOGGER.debug("Body contains image data.")
                else:
                    if response.context.options.get('stream', False):
                        _LOGGER.debug("Body is streamable")
                    else:
                        _LOGGER.debug(response.http_response.text())
        except Exception as err:  # pylint: disable=broad-except
            _LOGGER.debug("Failed to log response: %s", repr(err))


class RecordTelemetryUserAgentPolicy(UserAgentPolicy):
    def on_request(self, request):
        super().on_request(request)
        from azure.cli.core.telemetry import set_user_agent
        set_user_agent(request.http_request.headers[self._USERAGENT])


# pylint: disable=line-too-long
def get_custom_hook_policy(cli_ctx):
    def _acquire_policy_token_request_hook(request):
        http_request = request.http_request
        if getattr(http_request, 'method', '') == 'GET':
            return
        ACQUIRE_POLICY_TOKEN_URL = '/subscriptions/{subscriptionId}/providers/Microsoft.Authorization/acquirePolicyToken?api-version=2025-03-01'
        policy_token = None

        from azure.cli.core.azclierror import ServiceError
        try:
            import json
            from azure.cli.core.util import send_raw_request
            uri = getattr(http_request, 'url')
            method = getattr(http_request, 'method')
            content = getattr(http_request, 'content') if hasattr(http_request, 'content') else getattr(http_request, 'body')
            if content and isinstance(content, str):
                try:
                    content = json.loads(content)
                except json.decoder.JSONDecodeError:
                    pass

            acquire_policy_token_body = {
                "operation": {
                    "uri": uri,
                    "httpMethod": method,
                    "content": content
                },
                "changeReference": cli_ctx.data.get('_change_reference', None)
            }
            acquire_policy_token_response = send_raw_request(cli_ctx, 'POST',
                                                             ACQUIRE_POLICY_TOKEN_URL,
                                                             headers=['Content-Type=application/json',
                                                                      'x-ms-force-sync=true'],
                                                             body=json.dumps(acquire_policy_token_body))
            if acquire_policy_token_response.status_code == 200 and acquire_policy_token_response.content:
                response_content = json.loads(acquire_policy_token_response.content)
                policy_token = response_content.get('token', None)
                if not policy_token:
                    raise ServiceError(f"No token returned. Response:{acquire_policy_token_response.content}")
        except Exception as ex:
            raise ServiceError(f"Failed to acquire policy token, exception: {ex}")
        if policy_token:
            request.http_request.headers['x-ms-policy-external-evaluations'] = policy_token

    acquire_policy_token = cli_ctx.data.get('_acquire_policy_token', False)
    change_reference = cli_ctx.data.get('_change_reference', None)
    if change_reference or acquire_policy_token:
        from azure.core.pipeline.policies import CustomHookPolicy
        return CustomHookPolicy(raw_request_hook=_acquire_policy_token_request_hook)
    return None



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access, line-too-long

from ..aaz.latest.network.vnet_gateway.migration import Prepare as _VNetGatewayMigrationPrepare, \
    Execute as _VNetGatewayMigrationExecute, Commit as _VNetGatewayMigrationCommit, Abort as _VNetGatewayMigrationAbort


class VNetGatewayMigrationPrepare(_VNetGatewayMigrationPrepare):

    class VirtualNetworkGatewaysInvokePrepareMigration(_VNetGatewayMigrationPrepare.VirtualNetworkGatewaysInvokePrepareMigration):

        # to handle the LRO response with no response body
        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    lambda _: None,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)


class VNetGatewayMigrationExecute(_VNetGatewayMigrationExecute):

    class VirtualNetworkGatewaysInvokeExecuteMigration(_VNetGatewayMigrationExecute.VirtualNetworkGatewaysInvokeExecuteMigration):

        # to handle the LRO response with no response body
        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    lambda _: None,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)


class VNetGatewayMigrationCommit(_VNetGatewayMigrationCommit):

    class VirtualNetworkGatewaysInvokeCommitMigration(_VNetGatewayMigrationCommit.VirtualNetworkGatewaysInvokeCommitMigration):

        # to handle the LRO response with no response body
        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    lambda _: None,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)


class VNetGatewayMigrationAbort(_VNetGatewayMigrationAbort):

    class VirtualNetworkGatewaysInvokeAbortMigration(_VNetGatewayMigrationAbort.VirtualNetworkGatewaysInvokeAbortMigration):

        # to handle the LRO response with no response body
        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    lambda _: None,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "netappfiles volume show",
)
class Show(AAZCommand):
    """Get the details of the specified volume

    :example: Returns the properties of the given ANF volume
        az netappfiles volume show -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
    """

    _aaz_info = {
        "version": "2025-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.netapp/netappaccounts/{}/capacitypools/{}/volumes/{}", "2025-01-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.account_name = AAZStrArg(
            options=["-a", "--account-name"],
            help="The name of the NetApp account",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,127}$",
            ),
        )
        _args_schema.pool_name = AAZStrArg(
            options=["-p", "--pool-name"],
            help="The name of the capacity pool",
            required=True,
            id_part="child_name_1",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=1,
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.volume_name = AAZStrArg(
            options=["-n", "-v", "--name", "--volume-name"],
            help="The name of the volume",
            required=True,
            id_part="child_name_2",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=1,
            ),
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.VolumesGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class VolumesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.NetApp/netAppAccounts/{accountName}/capacityPools/{poolName}/volumes/{volumeName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "accountName", self.ctx.args.account_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "poolName", self.ctx.args.pool_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "volumeName", self.ctx.args.volume_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.zones = AAZListType()

            properties = cls._schema_on_200.properties
            properties.actual_throughput_mibps = AAZFloatType(
                serialized_name="actualThroughputMibps",
                flags={"read_only": True},
            )
            properties.avs_data_store = AAZStrType(
                serialized_name="avsDataStore",
            )
            properties.backup_id = AAZStrType(
                serialized_name="backupId",
                nullable=True,
            )
            properties.baremetal_tenant_id = AAZStrType(
                serialized_name="baremetalTenantId",
                flags={"read_only": True},
            )
            properties.capacity_pool_resource_id = AAZStrType(
                serialized_name="capacityPoolResourceId",
            )
            properties.clone_progress = AAZIntType(
                serialized_name="cloneProgress",
                nullable=True,
                flags={"read_only": True},
            )
            properties.cool_access = AAZBoolType(
                serialized_name="coolAccess",
            )
            properties.cool_access_retrieval_policy = AAZStrType(
                serialized_name="coolAccessRetrievalPolicy",
            )
            properties.cool_access_tiering_policy = AAZStrType(
                serialized_name="coolAccessTieringPolicy",
            )
            properties.coolness_period = AAZIntType(
                serialized_name="coolnessPeriod",
            )
            properties.creation_token = AAZStrType(
                serialized_name="creationToken",
                flags={"required": True},
            )
            properties.data_protection = AAZObjectType(
                serialized_name="dataProtection",
            )
            properties.data_store_resource_id = AAZListType(
                serialized_name="dataStoreResourceId",
                flags={"read_only": True},
            )
            properties.default_group_quota_in_ki_bs = AAZIntType(
                serialized_name="defaultGroupQuotaInKiBs",
            )
            properties.default_user_quota_in_ki_bs = AAZIntType(
                serialized_name="defaultUserQuotaInKiBs",
            )
            properties.delete_base_snapshot = AAZBoolType(
                serialized_name="deleteBaseSnapshot",
            )
            properties.effective_network_features = AAZStrType(
                serialized_name="effectiveNetworkFeatures",
            )
            properties.enable_subvolumes = AAZStrType(
                serialized_name="enableSubvolumes",
            )
            properties.encrypted = AAZBoolType(
                flags={"read_only": True},
            )
            properties.encryption_key_source = AAZStrType(
                serialized_name="encryptionKeySource",
            )
            properties.export_policy = AAZObjectType(
                serialized_name="exportPolicy",
            )
            properties.file_access_logs = AAZStrType(
                serialized_name="fileAccessLogs",
                flags={"read_only": True},
            )
            properties.file_system_id = AAZStrType(
                serialized_name="fileSystemId",
                flags={"read_only": True},
            )
            properties.is_default_quota_enabled = AAZBoolType(
                serialized_name="isDefaultQuotaEnabled",
            )
            properties.is_large_volume = AAZBoolType(
                serialized_name="isLargeVolume",
            )
            properties.is_restoring = AAZBoolType(
                serialized_name="isRestoring",
                flags={"read_only": True},
            )
            properties.kerberos_enabled = AAZBoolType(
                serialized_name="kerberosEnabled",
            )
            properties.key_vault_private_endpoint_resource_id = AAZStrType(
                serialized_name="keyVaultPrivateEndpointResourceId",
            )
            properties.ldap_enabled = AAZBoolType(
                serialized_name="ldapEnabled",
            )
            properties.maximum_number_of_files = AAZIntType(
                serialized_name="maximumNumberOfFiles",
                flags={"read_only": True},
            )
            properties.mount_targets = AAZListType(
                serialized_name="mountTargets",
                flags={"read_only": True},
            )
            properties.network_features = AAZStrType(
                serialized_name="networkFeatures",
            )
            properties.network_sibling_set_id = AAZStrType(
                serialized_name="networkSiblingSetId",
                flags={"read_only": True},
            )
            properties.originating_resource_id = AAZStrType(
                serialized_name="originatingResourceId",
                nullable=True,
                flags={"read_only": True},
            )
            properties.placement_rules = AAZListType(
                serialized_name="placementRules",
            )
            properties.protocol_types = AAZListType(
                serialized_name="protocolTypes",
            )
            properties.provisioned_availability_zone = AAZStrType(
                serialized_name="provisionedAvailabilityZone",
                nullable=True,
                flags={"read_only": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.proximity_placement_group = AAZStrType(
                serialized_name="proximityPlacementGroup",
            )
            properties.security_style = AAZStrType(
                serialized_name="securityStyle",
            )
            properties.service_level = AAZStrType(
                serialized_name="serviceLevel",
            )
            properties.smb_access_based_enumeration = AAZStrType(
                serialized_name="smbAccessBasedEnumeration",
                nullable=True,
            )
            properties.smb_continuously_available = AAZBoolType(
                serialized_name="smbContinuouslyAvailable",
            )
            properties.smb_encryption = AAZBoolType(
                serialized_name="smbEncryption",
            )
            properties.smb_non_browsable = AAZStrType(
                serialized_name="smbNonBrowsable",
            )
            properties.snapshot_directory_visible = AAZBoolType(
                serialized_name="snapshotDirectoryVisible",
            )
            properties.snapshot_id = AAZStrType(
                serialized_name="snapshotId",
                nullable=True,
            )
            properties.storage_to_network_proximity = AAZStrType(
                serialized_name="storageToNetworkProximity",
                flags={"read_only": True},
            )
            properties.subnet_id = AAZStrType(
                serialized_name="subnetId",
                flags={"required": True},
            )
            properties.t2_network = AAZStrType(
                serialized_name="t2Network",
                flags={"read_only": True},
            )
            properties.throughput_mibps = AAZFloatType(
                serialized_name="throughputMibps",
                nullable=True,
            )
            properties.unix_permissions = AAZStrType(
                serialized_name="unixPermissions",
                nullable=True,
            )
            properties.usage_threshold = AAZIntType(
                serialized_name="usageThreshold",
                flags={"required": True},
            )
            properties.volume_group_name = AAZStrType(
                serialized_name="volumeGroupName",
                flags={"read_only": True},
            )
            properties.volume_spec_name = AAZStrType(
                serialized_name="volumeSpecName",
            )
            properties.volume_type = AAZStrType(
                serialized_name="volumeType",
            )

            data_protection = cls._schema_on_200.properties.data_protection
            data_protection.backup = AAZObjectType()
            data_protection.replication = AAZObjectType()
            data_protection.snapshot = AAZObjectType()
            data_protection.volume_relocation = AAZObjectType(
                serialized_name="volumeRelocation",
            )

            backup = cls._schema_on_200.properties.data_protection.backup
            backup.backup_policy_id = AAZStrType(
                serialized_name="backupPolicyId",
            )
            backup.backup_vault_id = AAZStrType(
                serialized_name="backupVaultId",
            )
            backup.policy_enforced = AAZBoolType(
                serialized_name="policyEnforced",
            )

            replication = cls._schema_on_200.properties.data_protection.replication
            replication.destination_replications = AAZListType(
                serialized_name="destinationReplications",
                flags={"read_only": True},
            )
            replication.endpoint_type = AAZStrType(
                serialized_name="endpointType",
            )
            replication.remote_path = AAZObjectType(
                serialized_name="remotePath",
            )
            replication.remote_volume_region = AAZStrType(
                serialized_name="remoteVolumeRegion",
            )
            replication.remote_volume_resource_id = AAZStrType(
                serialized_name="remoteVolumeResourceId",
            )
            replication.replication_id = AAZStrType(
                serialized_name="replicationId",
                flags={"read_only": True},
            )
            replication.replication_schedule = AAZStrType(
                serialized_name="replicationSchedule",
            )

            destination_replications = cls._schema_on_200.properties.data_protection.replication.destination_replications
            destination_replications.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.data_protection.replication.destination_replications.Element
            _element.region = AAZStrType()
            _element.replication_type = AAZStrType(
                serialized_name="replicationType",
            )
            _element.resource_id = AAZStrType(
                serialized_name="resourceId",
            )
            _element.zone = AAZStrType()

            remote_path = cls._schema_on_200.properties.data_protection.replication.remote_path
            remote_path.external_host_name = AAZStrType(
                serialized_name="externalHostName",
                flags={"required": True},
            )
            remote_path.server_name = AAZStrType(
                serialized_name="serverName",
                flags={"required": True},
            )
            remote_path.volume_name = AAZStrType(
                serialized_name="volumeName",
                flags={"required": True},
            )

            snapshot = cls._schema_on_200.properties.data_protection.snapshot
            snapshot.snapshot_policy_id = AAZStrType(
                serialized_name="snapshotPolicyId",
            )

            volume_relocation = cls._schema_on_200.properties.data_protection.volume_relocation
            volume_relocation.ready_to_be_finalized = AAZBoolType(
                serialized_name="readyToBeFinalized",
                flags={"read_only": True},
            )
            volume_relocation.relocation_requested = AAZBoolType(
                serialized_name="relocationRequested",
            )

            data_store_resource_id = cls._schema_on_200.properties.data_store_resource_id
            data_store_resource_id.Element = AAZStrType()

            export_policy = cls._schema_on_200.properties.export_policy
            export_policy.rules = AAZListType()

            rules = cls._schema_on_200.properties.export_policy.rules
            rules.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.export_policy.rules.Element
            _element.allowed_clients = AAZStrType(
                serialized_name="allowedClients",
            )
            _element.chown_mode = AAZStrType(
                serialized_name="chownMode",
            )
            _element.cifs = AAZBoolType()
            _element.has_root_access = AAZBoolType(
                serialized_name="hasRootAccess",
            )
            _element.kerberos5_read_only = AAZBoolType(
                serialized_name="kerberos5ReadOnly",
            )
            _element.kerberos5_read_write = AAZBoolType(
                serialized_name="kerberos5ReadWrite",
            )
            _element.kerberos5i_read_only = AAZBoolType(
                serialized_name="kerberos5iReadOnly",
            )
            _element.kerberos5i_read_write = AAZBoolType(
                serialized_name="kerberos5iReadWrite",
            )
            _element.kerberos5p_read_only = AAZBoolType(
                serialized_name="kerberos5pReadOnly",
            )
            _element.kerberos5p_read_write = AAZBoolType(
                serialized_name="kerberos5pReadWrite",
            )
            _element.nfsv3 = AAZBoolType()
            _element.nfsv41 = AAZBoolType()
            _element.rule_index = AAZIntType(
                serialized_name="ruleIndex",
            )
            _element.unix_read_only = AAZBoolType(
                serialized_name="unixReadOnly",
            )
            _element.unix_read_write = AAZBoolType(
                serialized_name="unixReadWrite",
            )

            mount_targets = cls._schema_on_200.properties.mount_targets
            mount_targets.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.mount_targets.Element
            _element.file_system_id = AAZStrType(
                serialized_name="fileSystemId",
                flags={"required": True},
            )
            _element.ip_address = AAZStrType(
                serialized_name="ipAddress",
                flags={"read_only": True},
            )
            _element.mount_target_id = AAZStrType(
                serialized_name="mountTargetId",
                flags={"read_only": True},
            )
            _element.smb_server_fqdn = AAZStrType(
                serialized_name="smbServerFqdn",
            )

            placement_rules = cls._schema_on_200.properties.placement_rules
            placement_rules.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.placement_rules.Element
            _element.key = AAZStrType(
                flags={"required": True},
            )
            _element.value = AAZStrType(
                flags={"required": True},
            )

            protocol_types = cls._schema_on_200.properties.protocol_types
            protocol_types.Element = AAZStrType()

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            zones = cls._schema_on_200.zones
            zones.Element = AAZStrType()

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""


__all__ = ["Show"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "billing account list-invoice-section",
)
class ListInvoiceSection(AAZCommand):
    """List the invoice sections for which the user has permission to create Azure subscriptions. The operation is supported only for billing accounts with agreement type Microsoft Customer Agreement.
    """

    _aaz_info = {
        "version": "2020-05-01",
        "resources": [
            ["mgmt-plane", "/providers/microsoft.billing/billingaccounts/{}/listinvoicesectionswithcreatesubscriptionpermission", "2020-05-01"],
        ]
    }

    AZ_SUPPORT_PAGINATION = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_paging(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.billing_account_name = AAZStrArg(
            options=["--billing-account-name"],
            help="The ID that uniquely identifies a billing account.",
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.BillingAccountsListInvoiceSectionsByCreateSubscriptionPermission(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        next_link = self.deserialize_output(self.ctx.vars.instance.next_link)
        return result, next_link

    class BillingAccountsListInvoiceSectionsByCreateSubscriptionPermission(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/listInvoiceSectionsWithCreateSubscriptionPermission",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "billingAccountName", self.ctx.args.billing_account_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2020-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
                flags={"read_only": True},
            )
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.billing_profile_display_name = AAZStrType(
                serialized_name="billingProfileDisplayName",
                flags={"read_only": True},
            )
            _element.billing_profile_id = AAZStrType(
                serialized_name="billingProfileId",
                flags={"read_only": True},
            )
            _element.billing_profile_spending_limit = AAZStrType(
                serialized_name="billingProfileSpendingLimit",
                flags={"read_only": True},
            )
            _element.billing_profile_status = AAZStrType(
                serialized_name="billingProfileStatus",
                flags={"read_only": True},
            )
            _element.billing_profile_status_reason_code = AAZStrType(
                serialized_name="billingProfileStatusReasonCode",
                flags={"read_only": True},
            )
            _element.billing_profile_system_id = AAZStrType(
                serialized_name="billingProfileSystemId",
                flags={"read_only": True},
            )
            _element.enabled_azure_plans = AAZListType(
                serialized_name="enabledAzurePlans",
            )
            _element.invoice_section_display_name = AAZStrType(
                serialized_name="invoiceSectionDisplayName",
                flags={"read_only": True},
            )
            _element.invoice_section_id = AAZStrType(
                serialized_name="invoiceSectionId",
                flags={"read_only": True},
            )
            _element.invoice_section_system_id = AAZStrType(
                serialized_name="invoiceSectionSystemId",
                flags={"read_only": True},
            )

            enabled_azure_plans = cls._schema_on_200.value.Element.enabled_azure_plans
            enabled_azure_plans.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.enabled_azure_plans.Element
            _element.sku_description = AAZStrType(
                serialized_name="skuDescription",
                flags={"read_only": True},
            )
            _element.sku_id = AAZStrType(
                serialized_name="skuId",
            )

            return cls._schema_on_200


class _ListInvoiceSectionHelper:
    """Helper class for ListInvoiceSection"""


__all__ = ["ListInvoiceSection"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Update(AAZCommand):
    """Update a gallery in VM access control profile version.

    :example: Update a Gallery in VM access control profile version.
        az sig in-vm-access-control-profile-version update --resource-group myResourceGroup --gallery-name myGalleryName --profile-name myInVMAccessControlProfileName --profile-version 1.0.0 --exclude-from-latest true
    """

    _aaz_info = {
        "version": "2024-03-03",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/galleries/{}/invmaccesscontrolprofiles/{}/versions/{}", "2024-03-03"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.gallery_name = AAZStrArg(
            options=["--gallery-name"],
            help="The name of the Shared Image Gallery in which the in VM access control profile resides.",
            required=True,
            id_part="name",
        )
        _args_schema.profile_name = AAZStrArg(
            options=["--profile-name"],
            help="The name of the gallery in VM access control profile in which the in VM access control profile version is to be created.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.profile_version = AAZStrArg(
            options=["--version-name", "--profile-version"],
            help="The name of the gallery in VM access control profile version to be created. Needs to follow semantic version name pattern: The allowed characters are digit and period. Digits must be within the range of a 32-bit integer. Format: MajorVersion.MinorVersion.Patch",
            required=True,
            id_part="child_name_2",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "GalleryInVMAccessControlProfileVersion"

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.default_access = AAZStrArg(
            options=["--default-access"],
            arg_group="Properties",
            help="This property allows you to specify if the requests will be allowed to access the host endpoints. Possible values are: 'Allow', 'Deny'.",
            enum={"Allow": "Allow", "Deny": "Deny"},
        )
        _args_schema.exclude_from_latest = AAZBoolArg(
            options=["--exclude-from-latest"],
            arg_group="Properties",
            help="If set to true, Virtual Machines deployed from the latest version of the Resource Profile won't use this Profile version.",
            nullable=True,
        )
        _args_schema.mode = AAZStrArg(
            options=["--mode"],
            arg_group="Properties",
            help="This property allows you to specify whether the access control rules are in Audit mode, in Enforce mode or Disabled. Possible values are: 'Audit', 'Enforce' or 'Disabled'.",
            enum={"Audit": "Audit", "Disabled": "Disabled", "Enforce": "Enforce"},
        )
        _args_schema.rules = AAZObjectArg(
            options=["--rules"],
            arg_group="Properties",
            help="This is the Access Control Rules specification for an in VM access control profile version.",
            nullable=True,
        )
        _args_schema.target_locations = AAZListArg(
            options=["--target-locations"],
            arg_group="Properties",
            help="The target regions where the Resource Profile version is going to be replicated to. This property is updatable.",
            nullable=True,
        )

        rules = cls._args_schema.rules
        rules.identities = AAZListArg(
            options=["identities"],
            help="A list of identities.",
            nullable=True,
        )
        rules.privileges = AAZListArg(
            options=["privileges"],
            help="A list of privileges.",
            nullable=True,
        )
        rules.role_assignments = AAZListArg(
            options=["role-assignments"],
            help="A list of role assignments.",
            nullable=True,
        )
        rules.roles = AAZListArg(
            options=["roles"],
            help="A list of roles.",
            nullable=True,
        )

        identities = cls._args_schema.rules.identities
        identities.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.rules.identities.Element
        _element.exe_path = AAZStrArg(
            options=["exe-path"],
            help="The path to the executable.",
            nullable=True,
        )
        _element.group_name = AAZStrArg(
            options=["group-name"],
            help="The groupName corresponding to this identity.",
            nullable=True,
        )
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the identity.",
        )
        _element.process_name = AAZStrArg(
            options=["process-name"],
            help="The process name of the executable.",
            nullable=True,
        )
        _element.user_name = AAZStrArg(
            options=["user-name"],
            help="The username corresponding to this identity.",
            nullable=True,
        )

        privileges = cls._args_schema.rules.privileges
        privileges.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.rules.privileges.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the privilege.",
        )
        _element.path = AAZStrArg(
            options=["path"],
            help="The HTTP path corresponding to the privilege.",
        )
        _element.query_parameters = AAZDictArg(
            options=["query-parameters"],
            help="The query parameters to match in the path.",
            nullable=True,
        )

        query_parameters = cls._args_schema.rules.privileges.Element.query_parameters
        query_parameters.Element = AAZStrArg(
            nullable=True,
        )

        role_assignments = cls._args_schema.rules.role_assignments
        role_assignments.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.rules.role_assignments.Element
        _element.identities = AAZListArg(
            options=["identities"],
            help="A list of identities that can access the privileges defined by the role.",
        )
        _element.role = AAZStrArg(
            options=["role"],
            help="The name of the role.",
        )

        identities = cls._args_schema.rules.role_assignments.Element.identities
        identities.Element = AAZStrArg(
            nullable=True,
        )

        roles = cls._args_schema.rules.roles
        roles.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.rules.roles.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the role.",
        )
        _element.privileges = AAZListArg(
            options=["privileges"],
            help="A list of privileges needed by this role.",
        )

        privileges = cls._args_schema.rules.roles.Element.privileges
        privileges.Element = AAZStrArg(
            nullable=True,
        )

        target_locations = cls._args_schema.target_locations
        target_locations.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.target_locations.Element
        _element.additional_replica_sets = AAZListArg(
            options=["additional-replica-sets"],
            help="List of storage sku with replica count to create direct drive replicas.",
            nullable=True,
        )
        _element.encryption = AAZObjectArg(
            options=["encryption"],
            help="Optional. Allows users to provide customer managed keys for encrypting the OS and data disks in the gallery artifact.",
            nullable=True,
        )
        _element.exclude_from_latest = AAZBoolArg(
            options=["exclude-from-latest"],
            help="Contains the flag setting to hide an image when users specify version='latest'",
            nullable=True,
        )
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the region.",
        )
        _element.regional_replica_count = AAZIntArg(
            options=["regional-replica-count"],
            help="The number of replicas of the Image Version to be created per region. This property is updatable.",
            nullable=True,
        )
        _element.storage_account_type = AAZStrArg(
            options=["storage-account-type"],
            help="Specifies the storage account type to be used to store the image. This property is not updatable.",
            nullable=True,
            enum={"PremiumV2_LRS": "PremiumV2_LRS", "Premium_LRS": "Premium_LRS", "Standard_LRS": "Standard_LRS", "Standard_ZRS": "Standard_ZRS"},
        )

        additional_replica_sets = cls._args_schema.target_locations.Element.additional_replica_sets
        additional_replica_sets.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.target_locations.Element.additional_replica_sets.Element
        _element.regional_replica_count = AAZIntArg(
            options=["regional-replica-count"],
            help="The number of direct drive replicas of the Image Version to be created.This Property is updatable",
            nullable=True,
        )
        _element.storage_account_type = AAZStrArg(
            options=["storage-account-type"],
            help="Specifies the storage account type to be used to create the direct drive replicas",
            nullable=True,
            enum={"PremiumV2_LRS": "PremiumV2_LRS", "Premium_LRS": "Premium_LRS", "Standard_LRS": "Standard_LRS", "Standard_ZRS": "Standard_ZRS"},
        )

        encryption = cls._args_schema.target_locations.Element.encryption
        encryption.data_disk_images = AAZListArg(
            options=["data-disk-images"],
            help="A list of encryption specifications for data disk images.",
            nullable=True,
        )
        encryption.os_disk_image = AAZObjectArg(
            options=["os-disk-image"],
            help="Contains encryption settings for an OS disk image.",
            nullable=True,
        )

        data_disk_images = cls._args_schema.target_locations.Element.encryption.data_disk_images
        data_disk_images.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.target_locations.Element.encryption.data_disk_images.Element
        _element.disk_encryption_set_id = AAZStrArg(
            options=["disk-encryption-set-id"],
            help="A relative URI containing the resource ID of the disk encryption set.",
            nullable=True,
        )
        _element.lun = AAZIntArg(
            options=["lun"],
            help="This property specifies the logical unit number of the data disk. This value is used to identify data disks within the Virtual Machine and therefore must be unique for each data disk attached to the Virtual Machine.",
        )

        os_disk_image = cls._args_schema.target_locations.Element.encryption.os_disk_image
        os_disk_image.disk_encryption_set_id = AAZStrArg(
            options=["disk-encryption-set-id"],
            help="A relative URI containing the resource ID of the disk encryption set.",
            nullable=True,
        )
        os_disk_image.security_profile = AAZObjectArg(
            options=["security-profile"],
            help="This property specifies the security profile of an OS disk image.",
            nullable=True,
        )

        security_profile = cls._args_schema.target_locations.Element.encryption.os_disk_image.security_profile
        security_profile.confidential_vm_encryption_type = AAZStrArg(
            options=["confidential-vm-encryption-type"],
            help="confidential VM encryption types",
            nullable=True,
            enum={"EncryptedVMGuestStateOnlyWithPmk": "EncryptedVMGuestStateOnlyWithPmk", "EncryptedWithCmk": "EncryptedWithCmk", "EncryptedWithPmk": "EncryptedWithPmk", "NonPersistedTPM": "NonPersistedTPM"},
        )
        security_profile.secure_vm_disk_encryption_set_id = AAZStrArg(
            options=["secure-vm-disk-encryption-set-id"],
            help="secure VM disk encryption set id",
            nullable=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.GalleryInVMAccessControlProfileVersionsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.GalleryInVMAccessControlProfileVersionsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class GalleryInVMAccessControlProfileVersionsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/galleries/{galleryName}/inVMAccessControlProfiles/{inVMAccessControlProfileName}/versions/{inVMAccessControlProfileVersionName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "galleryName", self.ctx.args.gallery_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "inVMAccessControlProfileName", self.ctx.args.profile_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "inVMAccessControlProfileVersionName", self.ctx.args.profile_version,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-03-03",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_gallery_in_vm_access_control_profile_version_read(cls._schema_on_200)

            return cls._schema_on_200

    class GalleryInVMAccessControlProfileVersionsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/galleries/{galleryName}/inVMAccessControlProfiles/{inVMAccessControlProfileName}/versions/{inVMAccessControlProfileVersionName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "galleryName", self.ctx.args.gallery_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "inVMAccessControlProfileName", self.ctx.args.profile_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "inVMAccessControlProfileVersionName", self.ctx.args.profile_version,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-03-03",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_gallery_in_vm_access_control_profile_version_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("defaultAccess", AAZStrType, ".default_access", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("excludeFromLatest", AAZBoolType, ".exclude_from_latest")
                properties.set_prop("mode", AAZStrType, ".mode", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("rules", AAZObjectType, ".rules")
                properties.set_prop("targetLocations", AAZListType, ".target_locations")

            rules = _builder.get(".properties.rules")
            if rules is not None:
                rules.set_prop("identities", AAZListType, ".identities")
                rules.set_prop("privileges", AAZListType, ".privileges")
                rules.set_prop("roleAssignments", AAZListType, ".role_assignments")
                rules.set_prop("roles", AAZListType, ".roles")

            identities = _builder.get(".properties.rules.identities")
            if identities is not None:
                identities.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.rules.identities[]")
            if _elements is not None:
                _elements.set_prop("exePath", AAZStrType, ".exe_path")
                _elements.set_prop("groupName", AAZStrType, ".group_name")
                _elements.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("processName", AAZStrType, ".process_name")
                _elements.set_prop("userName", AAZStrType, ".user_name")

            privileges = _builder.get(".properties.rules.privileges")
            if privileges is not None:
                privileges.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.rules.privileges[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("path", AAZStrType, ".path", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("queryParameters", AAZDictType, ".query_parameters")

            query_parameters = _builder.get(".properties.rules.privileges[].queryParameters")
            if query_parameters is not None:
                query_parameters.set_elements(AAZStrType, ".")

            role_assignments = _builder.get(".properties.rules.roleAssignments")
            if role_assignments is not None:
                role_assignments.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.rules.roleAssignments[]")
            if _elements is not None:
                _elements.set_prop("identities", AAZListType, ".identities", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("role", AAZStrType, ".role", typ_kwargs={"flags": {"required": True}})

            identities = _builder.get(".properties.rules.roleAssignments[].identities")
            if identities is not None:
                identities.set_elements(AAZStrType, ".")

            roles = _builder.get(".properties.rules.roles")
            if roles is not None:
                roles.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.rules.roles[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("privileges", AAZListType, ".privileges", typ_kwargs={"flags": {"required": True}})

            privileges = _builder.get(".properties.rules.roles[].privileges")
            if privileges is not None:
                privileges.set_elements(AAZStrType, ".")

            target_locations = _builder.get(".properties.targetLocations")
            if target_locations is not None:
                target_locations.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.targetLocations[]")
            if _elements is not None:
                _elements.set_prop("additionalReplicaSets", AAZListType, ".additional_replica_sets")
                _elements.set_prop("encryption", AAZObjectType, ".encryption")
                _elements.set_prop("excludeFromLatest", AAZBoolType, ".exclude_from_latest")
                _elements.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("regionalReplicaCount", AAZIntType, ".regional_replica_count")
                _elements.set_prop("storageAccountType", AAZStrType, ".storage_account_type")

            additional_replica_sets = _builder.get(".properties.targetLocations[].additionalReplicaSets")
            if additional_replica_sets is not None:
                additional_replica_sets.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.targetLocations[].additionalReplicaSets[]")
            if _elements is not None:
                _elements.set_prop("regionalReplicaCount", AAZIntType, ".regional_replica_count")
                _elements.set_prop("storageAccountType", AAZStrType, ".storage_account_type")

            encryption = _builder.get(".properties.targetLocations[].encryption")
            if encryption is not None:
                encryption.set_prop("dataDiskImages", AAZListType, ".data_disk_images")
                encryption.set_prop("osDiskImage", AAZObjectType, ".os_disk_image")

            data_disk_images = _builder.get(".properties.targetLocations[].encryption.dataDiskImages")
            if data_disk_images is not None:
                data_disk_images.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.targetLocations[].encryption.dataDiskImages[]")
            if _elements is not None:
                _elements.set_prop("diskEncryptionSetId", AAZStrType, ".disk_encryption_set_id")
                _elements.set_prop("lun", AAZIntType, ".lun", typ_kwargs={"flags": {"required": True}})

            os_disk_image = _builder.get(".properties.targetLocations[].encryption.osDiskImage")
            if os_disk_image is not None:
                os_disk_image.set_prop("diskEncryptionSetId", AAZStrType, ".disk_encryption_set_id")
                os_disk_image.set_prop("securityProfile", AAZObjectType, ".security_profile")

            security_profile = _builder.get(".properties.targetLocations[].encryption.osDiskImage.securityProfile")
            if security_profile is not None:
                security_profile.set_prop("confidentialVMEncryptionType", AAZStrType, ".confidential_vm_encryption_type")
                security_profile.set_prop("secureVMDiskEncryptionSetId", AAZStrType, ".secure_vm_disk_encryption_set_id")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    _schema_gallery_in_vm_access_control_profile_version_read = None

    @classmethod
    def _build_schema_gallery_in_vm_access_control_profile_version_read(cls, _schema):
        if cls._schema_gallery_in_vm_access_control_profile_version_read is not None:
            _schema.id = cls._schema_gallery_in_vm_access_control_profile_version_read.id
            _schema.location = cls._schema_gallery_in_vm_access_control_profile_version_read.location
            _schema.name = cls._schema_gallery_in_vm_access_control_profile_version_read.name
            _schema.properties = cls._schema_gallery_in_vm_access_control_profile_version_read.properties
            _schema.tags = cls._schema_gallery_in_vm_access_control_profile_version_read.tags
            _schema.type = cls._schema_gallery_in_vm_access_control_profile_version_read.type
            return

        cls._schema_gallery_in_vm_access_control_profile_version_read = _schema_gallery_in_vm_access_control_profile_version_read = AAZObjectType()

        gallery_in_vm_access_control_profile_version_read = _schema_gallery_in_vm_access_control_profile_version_read
        gallery_in_vm_access_control_profile_version_read.id = AAZStrType(
            flags={"read_only": True},
        )
        gallery_in_vm_access_control_profile_version_read.location = AAZStrType(
            flags={"required": True},
        )
        gallery_in_vm_access_control_profile_version_read.name = AAZStrType(
            flags={"read_only": True},
        )
        gallery_in_vm_access_control_profile_version_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        gallery_in_vm_access_control_profile_version_read.tags = AAZDictType()
        gallery_in_vm_access_control_profile_version_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_gallery_in_vm_access_control_profile_version_read.properties
        properties.default_access = AAZStrType(
            serialized_name="defaultAccess",
            flags={"required": True},
        )
        properties.exclude_from_latest = AAZBoolType(
            serialized_name="excludeFromLatest",
        )
        properties.mode = AAZStrType(
            flags={"required": True},
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.published_date = AAZStrType(
            serialized_name="publishedDate",
            flags={"read_only": True},
        )
        properties.replication_status = AAZObjectType(
            serialized_name="replicationStatus",
            flags={"read_only": True},
        )
        properties.rules = AAZObjectType()
        properties.target_locations = AAZListType(
            serialized_name="targetLocations",
        )

        replication_status = _schema_gallery_in_vm_access_control_profile_version_read.properties.replication_status
        replication_status.aggregated_state = AAZStrType(
            serialized_name="aggregatedState",
            flags={"read_only": True},
        )
        replication_status.summary = AAZListType(
            flags={"read_only": True},
        )

        summary = _schema_gallery_in_vm_access_control_profile_version_read.properties.replication_status.summary
        summary.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.replication_status.summary.Element
        _element.details = AAZStrType(
            flags={"read_only": True},
        )
        _element.progress = AAZIntType(
            flags={"read_only": True},
        )
        _element.region = AAZStrType(
            flags={"read_only": True},
        )
        _element.state = AAZStrType(
            flags={"read_only": True},
        )

        rules = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules
        rules.identities = AAZListType()
        rules.privileges = AAZListType()
        rules.role_assignments = AAZListType(
            serialized_name="roleAssignments",
        )
        rules.roles = AAZListType()

        identities = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.identities
        identities.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.identities.Element
        _element.exe_path = AAZStrType(
            serialized_name="exePath",
        )
        _element.group_name = AAZStrType(
            serialized_name="groupName",
        )
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.process_name = AAZStrType(
            serialized_name="processName",
        )
        _element.user_name = AAZStrType(
            serialized_name="userName",
        )

        privileges = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.privileges
        privileges.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.privileges.Element
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.path = AAZStrType(
            flags={"required": True},
        )
        _element.query_parameters = AAZDictType(
            serialized_name="queryParameters",
        )

        query_parameters = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.privileges.Element.query_parameters
        query_parameters.Element = AAZStrType()

        role_assignments = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.role_assignments
        role_assignments.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.role_assignments.Element
        _element.identities = AAZListType(
            flags={"required": True},
        )
        _element.role = AAZStrType(
            flags={"required": True},
        )

        identities = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.role_assignments.Element.identities
        identities.Element = AAZStrType()

        roles = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.roles
        roles.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.roles.Element
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.privileges = AAZListType(
            flags={"required": True},
        )

        privileges = _schema_gallery_in_vm_access_control_profile_version_read.properties.rules.roles.Element.privileges
        privileges.Element = AAZStrType()

        target_locations = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations
        target_locations.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element
        _element.additional_replica_sets = AAZListType(
            serialized_name="additionalReplicaSets",
        )
        _element.encryption = AAZObjectType()
        _element.exclude_from_latest = AAZBoolType(
            serialized_name="excludeFromLatest",
        )
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.regional_replica_count = AAZIntType(
            serialized_name="regionalReplicaCount",
        )
        _element.storage_account_type = AAZStrType(
            serialized_name="storageAccountType",
        )

        additional_replica_sets = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.additional_replica_sets
        additional_replica_sets.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.additional_replica_sets.Element
        _element.regional_replica_count = AAZIntType(
            serialized_name="regionalReplicaCount",
        )
        _element.storage_account_type = AAZStrType(
            serialized_name="storageAccountType",
        )

        encryption = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.encryption
        encryption.data_disk_images = AAZListType(
            serialized_name="dataDiskImages",
        )
        encryption.os_disk_image = AAZObjectType(
            serialized_name="osDiskImage",
        )

        data_disk_images = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.encryption.data_disk_images
        data_disk_images.Element = AAZObjectType()

        _element = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.encryption.data_disk_images.Element
        _element.disk_encryption_set_id = AAZStrType(
            serialized_name="diskEncryptionSetId",
        )
        _element.lun = AAZIntType(
            flags={"required": True},
        )

        os_disk_image = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.encryption.os_disk_image
        os_disk_image.disk_encryption_set_id = AAZStrType(
            serialized_name="diskEncryptionSetId",
        )
        os_disk_image.security_profile = AAZObjectType(
            serialized_name="securityProfile",
        )

        security_profile = _schema_gallery_in_vm_access_control_profile_version_read.properties.target_locations.Element.encryption.os_disk_image.security_profile
        security_profile.confidential_vm_encryption_type = AAZStrType(
            serialized_name="confidentialVMEncryptionType",
        )
        security_profile.secure_vm_disk_encryption_set_id = AAZStrType(
            serialized_name="secureVMDiskEncryptionSetId",
        )

        tags = _schema_gallery_in_vm_access_control_profile_version_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_gallery_in_vm_access_control_profile_version_read.id
        _schema.location = cls._schema_gallery_in_vm_access_control_profile_version_read.location
        _schema.name = cls._schema_gallery_in_vm_access_control_profile_version_read.name
        _schema.properties = cls._schema_gallery_in_vm_access_control_profile_version_read.properties
        _schema.tags = cls._schema_gallery_in_vm_access_control_profile_version_read.tags
        _schema.type = cls._schema_gallery_in_vm_access_control_profile_version_read.type


__all__ = ["Update"]
